import unittest

from conans import tools
from conans.test.utils.tools import TestBufferConanOutput
from cpt.auth import AuthManager
from cpt.printer import Printer
from cpt.test.unit.packager_test import MockConanAPI


class AuthTest(unittest.TestCase):

    def setUp(self):
        self.conan_api = MockConanAPI()
        self.output = TestBufferConanOutput()
        self.printer = Printer(self.output.write)

    def no_credentials_test(self):
        manager = AuthManager(self.conan_api, self.printer)
        user, password = manager.get_user_password()
        self.assertEquals(user, None)
        self.assertEquals(password, None)

    def test_plain_credentials(self):

        # Without default
        manager = AuthManager(self.conan_api, self.printer, login_input="myuser",
                              passwords_input="mypassword")

        user, password = manager.get_user_password("any")
        self.assertEquals(user, "myuser")
        self.assertEquals(password, "mypassword")

        user, password = manager.get_user_password(None)
        self.assertEquals(user, "myuser")
        self.assertEquals(password, "mypassword")

        # Only password is discarded
        manager = AuthManager(self.conan_api, self.printer, passwords_input="mypassword")
        user, password = manager.get_user_password()
        self.assertEquals(user, None)
        self.assertEquals(password, None)

        # With default
        manager = AuthManager(self.conan_api, self.printer,
                              passwords_input="mypassword",
                              default_username="myuser")

        user, password = manager.get_user_password("any")
        self.assertEquals(user, "myuser")
        self.assertEquals(password, "mypassword")

    def plain_from_env_test(self):
        with tools.environment_append({"CONAN_LOGIN_USERNAME": "myuser",
                                       "CONAN_PASSWORD": "mypass"}):
            manager = AuthManager(self.conan_api, self.printer)
            user, password = manager.get_user_password()
            self.assertEquals(user, "myuser")
            self.assertEquals(password, "mypass")

    def plain_multiple_from_env_test(self):
        # Bad mix
        with tools.environment_append({"CONAN_LOGIN_USERNAME_R1": "myuser",
                                       "CONAN_PASSWORD": "mypass"}):
            with self.assertRaisesRegexp(Exception, "Password for remote 'R1' not specified"):
                AuthManager(self.conan_api, self.printer)

        with tools.environment_append({"CONAN_LOGIN_USERNAME_R1": "myuser",
                                       "CONAN_PASSWORD_R1": "mypass",
                                       "CONAN_LOGIN_USERNAME_R_OTHER": "myuser2",
                                       "CONAN_PASSWORD_R_OTHER": "mypass2"}):
            manager = AuthManager(self.conan_api, self.printer)
            user, password = manager.get_user_password("r1")
            self.assertEquals(user, "myuser")
            self.assertEquals(password, "mypass")

            user, password = manager.get_user_password("r_other")
            self.assertEquals(user, "myuser2")
            self.assertEquals(password, "mypass2")

        # Miss password
        with tools.environment_append({"CONAN_LOGIN_USERNAME_R1": "myuser",
                                       "CONAN_PASSWORD_R2": "mypass"}):
            with self.assertRaisesRegexp(Exception, "Password for remote 'R1' not specified"):
                AuthManager(self.conan_api, self.printer)

    def plain_from_env_priority_test(self):
        with tools.environment_append({"CONAN_LOGIN_USERNAME": "myuser",
                                       "CONAN_PASSWORD": "mypass"}):
            manager = AuthManager(self.conan_api, self.printer, login_input="otheruser",
                                  passwords_input="otherpass")
            user, password = manager.get_user_password()
            self.assertEquals(user, "otheruser")
            self.assertEquals(password, "otherpass")

    def plain_from_env_priority_mix_test(self):
        with tools.environment_append({"CONAN_LOGIN_USERNAME": "myuser",
                                       "CONAN_PASSWORD": "mypass"}):
            manager = AuthManager(self.conan_api, self.printer, login_input="otheruser")
            user, password = manager.get_user_password()
            self.assertEquals(user, "otheruser")
            self.assertEquals(password, "mypass")

    def test_dict_credentials(self):
        users = {"remote1": "my_user", "my_artifactory": "other_user"}
        passwords = {"remote1": "my_pass", "my_artifactory": "my_pass2"}
        manager = AuthManager(self.conan_api, self.printer, login_input=users,
                              passwords_input=passwords,
                              default_username=None)

        with self.assertRaisesRegexp(Exception, "User and password for remote "
                                                "'not_exist' not specified"):
            manager.get_user_password("not_exist")

        user, password = manager.get_user_password("my_artifactory")
        self.assertEquals(user, "other_user")
        self.assertEquals(password, "my_pass2")

        user, password = manager.get_user_password("remote1")
        self.assertEquals(user, "my_user")
        self.assertEquals(password, "my_pass")

        # Mix them
        with self.assertRaisesRegexp(Exception, "Specify a dict for 'login_username'"):
            AuthManager(self.conan_api, self.printer, passwords_input=passwords, default_username="peter")

    def test_env_vars_output(self):
        users = {"remote1": "my_user", "my_artifactory": "other_user"}
        passwords = {"remote1": "my_pass", "my_artifactory": "my_pass2"}
        manager = AuthManager(self.conan_api, self.printer, login_input=users,
                              passwords_input=passwords)
        expected = {'CONAN_PASSWORD_REMOTE1': 'my_pass',
                    'CONAN_LOGIN_USERNAME_REMOTE1': 'my_user',
                    'CONAN_PASSWORD_MY_ARTIFACTORY': 'my_pass2',
                    'CONAN_LOGIN_USERNAME_MY_ARTIFACTORY': 'other_user'}
        self.assertEquals(manager.env_vars(), expected)

        with tools.environment_append(expected):
            manager = AuthManager(self.conan_api, self.printer)
            self.assertEquals(manager.env_vars(), expected)

        manager = AuthManager(self.conan_api, self.printer, login_input="myuser",
                              passwords_input="mypassword")
        expected = {'CONAN_PASSWORD': 'mypassword',
                    'CONAN_LOGIN_USERNAME': 'myuser'}

        self.assertEquals(manager.env_vars(), expected)



import subprocess
import unittest
import time

from conans import tools
from conans.model.ref import ConanFileReference
from conans.model.version import Version
from cpt import get_client_version
from cpt.packager import ConanMultiPackager
from cpt.test.integration.base import BaseTest, CONAN_UPLOAD_PASSWORD, CONAN_LOGIN_UPLOAD
from cpt.test.unit.utils import MockCIManager


def is_linux_and_have_docker():
    return tools.os_info.is_linux and tools.which("docker")


class DockerTest(BaseTest):

    CONAN_SERVER_ADDRESS = "http://0.0.0.0:9300"

    def setUp(self):
        super(DockerTest, self).setUp()
        self.server_process = subprocess.Popen("conan_server")
        time.sleep(3)

    def tearDown(self):
        self.server_process.kill()
        super(DockerTest, self).tearDown()

    @unittest.skipUnless(is_linux_and_have_docker(), "Requires Linux and Docker")
    def test_docker(self):
        client_version = get_client_version()
        ci_manager = MockCIManager()
        unique_ref = "zlib/%s" % str(time.time())
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

"""

        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_DOCKER_RUN_OPTIONS": "--network=host -v{}:/tmp/cpt".format(self.root_project_folder),
                                       "CONAN_DOCKER_ENTRY_SCRIPT": "pip install -U /tmp/cpt",
                                       "CONAN_USE_DOCKER": "1",
                                       "CONAN_DOCKER_IMAGE_SKIP_UPDATE": "TRUE",
                                       "CONAN_LOGIN_USERNAME": "demo",
                                       "CONAN_USERNAME": "demo",
                                       "CONAN_UPLOAD": DockerTest.CONAN_SERVER_ADDRESS,
                                       "CONAN_PASSWORD": "demo"}):

            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               reference=unique_ref,
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        search_pattern = "%s*" % unique_ref
        ref = ConanFileReference.loads("%s@demo/mychannel" % unique_ref)

        # Remove from remote
        if Version(client_version) < Version("1.7"):
            results = self.api.search_recipes(search_pattern, remote="upload_repo")["results"][0]["items"]
            self.assertEquals(len(results), 1)
            packages = self.api.search_packages(ref, remote="upload_repo")["results"][0]["items"][0]["packages"]
            self.assertEquals(len(packages), 2)
            self.api.authenticate(name=CONAN_LOGIN_UPLOAD, password=CONAN_UPLOAD_PASSWORD,
                                  remote="upload_repo")
            self.api.remove(search_pattern, remote="upload_repo", force=True)
            self.assertEquals(self.api.search_recipes(search_pattern)["results"], [])
        else:
            results = self.api.search_recipes(search_pattern, remote_name="upload_repo")["results"][0]["items"]
            self.assertEquals(len(results), 1)
            if Version(client_version) >= Version("1.12.0"):
                ref = repr(ref)
            packages = self.api.search_packages(ref, remote_name="upload_repo")["results"][0]["items"][0]["packages"]
            self.assertEquals(len(packages), 2)
            self.api.authenticate(name="demo", password="demo",
                                  remote_name="upload_repo")
            self.api.remove(search_pattern, remote_name="upload_repo", force=True)
            self.assertEquals(self.api.search_recipes(search_pattern)["results"], [])

        # Try upload only when stable, shouldn't upload anything
        with tools.environment_append({"CONAN_DOCKER_RUN_OPTIONS": "--network=host -v{}:/tmp/cpt".format(self.root_project_folder),
                                       "CONAN_DOCKER_ENTRY_SCRIPT": "pip install -U /tmp/cpt",
                                       "CONAN_USE_DOCKER": "1",
                                       "CONAN_LOGIN_USERNAME": "demo",
                                       "CONAN_USERNAME": "demo",
                                       "CONAN_PASSWORD": "demo",
                                       "CONAN_DOCKER_IMAGE_SKIP_UPDATE": "TRUE",
                                       "CONAN_UPLOAD_ONLY_WHEN_STABLE": "1"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               reference=unique_ref,
                                               upload=DockerTest.CONAN_SERVER_ADDRESS,
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        if Version(client_version) < Version("1.7"):
            results = self.api.search_recipes(search_pattern, remote="upload_repo")["results"]
            self.assertEquals(len(results), 0)
            self.api.remove(search_pattern, remote="upload_repo", force=True)
        else:
            results = self.api.search_recipes(search_pattern, remote_name="upload_repo")["results"]
            self.assertEquals(len(results), 0)
            self.api.remove(search_pattern, remote_name="upload_repo", force=True)

    @unittest.skipUnless(is_linux_and_have_docker(), "Requires Linux and Docker")
    def test_docker_run_options(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    requires = "zlib/1.2.11@conan/stable"

    def build(self):
        pass
"""
        self.save_conanfile(conanfile)
        # Validate by Environemnt Variable
        with tools.environment_append({"CONAN_DOCKER_ENTRY_SCRIPT": "pip install -U /tmp/cpt",
                                       "CONAN_USERNAME": "bar",
                                       "CONAN_DOCKER_IMAGE": "conanio/gcc8",
                                       "CONAN_REFERENCE": "foo/0.0.1@bar/testing",
                                       "CONAN_DOCKER_RUN_OPTIONS": "--network=host, --add-host=google.com:8.8.8.8 -v{}:/tmp/cpt".format(self.root_project_folder),
                                       "CONAN_DOCKER_IMAGE_SKIP_UPDATE": "TRUE",
                                       "CONAN_FORCE_SELINUX": "TRUE",
                                       "CONAN_DOCKER_SHELL": "/bin/bash -c"
                                       }):
            self.packager = ConanMultiPackager(gcc_versions=["8"],
                                               archs=["x86_64"],
                                               build_types=["Release"],
                                               out=self.output.write)
            self.packager.add({})
            self.packager.run()
            self.assertIn("--network=host --add-host=google.com:8.8.8.8 -v", self.output)
            self.assertIn("/bin/bash -c", self.output)
            self.assertIn("/home/conan/project:z", self.output)

        # Validate by parameter
        with tools.environment_append({"CONAN_USERNAME": "bar",
                                       "CONAN_DOCKER_IMAGE": "conanio/gcc8",
                                       "CONAN_REFERENCE": "foo/0.0.1@bar/testing",

                                       }):

            self.packager = ConanMultiPackager(gcc_versions=["8"],
                                               archs=["x86_64"],
                                               build_types=["Release"],
                                               docker_run_options="--network=host -v{}:/tmp/cpt --cpus=1".format(self.root_project_folder) ,
                                               docker_entry_script="pip install -U /tmp/cpt",
                                               docker_image_skip_update=True,
                                               docker_shell="/bin/bash -c",
                                               out=self.output.write,
                                               force_selinux=True)
            self.packager.add({})
            self.packager.run()
            self.assertIn("--cpus=1  conanio/gcc8", self.output)
            self.assertIn("/bin/bash -c", self.output)
            self.assertIn("/home/conan/project:z", self.output)

    @unittest.skipUnless(is_linux_and_have_docker(), "Requires Linux and Docker")
    def test_docker_run_android(self):
        self.create_project()
        command = ('docker run --rm -v "{}:/home/conan/project" ',
                   '-e CONAN_RECIPE_LINTER="False" ',
                   '-e CONAN_PIP_PACKAGE="0" ',
                   '-e CONAN_DOCKER_ENTRY_SCRIPT="pip install -U /tmp/cpt" ',
                   '-e CONAN_USERNAME="bar" ',
                   '-e CONAN_DOCKER_IMAGE="conanio/android-clang8" ',
                   '-e CONAN_CHANNEL="testing" ',
                   '-e CONAN_DOCKER_RUN_OPTIONS="-v{}:/tmp/cpt" ',
                   '-e CONAN_DOCKER_IMAGE_SKIP_UPDATE="TRUE" ',
                   '-e CONAN_DOCKER_USE_SUDO="FALSE" ',
                   '-e CONAN_ARCHS="x86_64" ',
                   '-e CONAN_CLANG_VERSIONS="8" ',
                   '-e CONAN_BUILD_TYPES="Release" ',
                   '-e CONAN_LOGIN_USERNAME="bar" ',
                   '-e CONAN_REFERENCE="hello/0.1.0@bar/testing" ',
                   '-e CPT_PROFILE="@@include(default)@@@@[settings]@@arch=x86_64@@build_type=Release@@compiler=clang@@compiler.version=8@@[options]@@@@[env]@@@@[build_requires]@@@@" ',
                   '-e CONAN_TEMP_TEST_FOLDER="1" ',
                   '-e CPT_UPLOAD_RETRY="3" ',
                   '-e CPT_CONANFILE="conanfile.py" ',
                   '-v{}:/tmp/cpt  ',
                   'conanio/android-clang8 ',
                   '/bin/sh -c " cd project &&  pip install -U /tmp/cpt && run_create_in_docker "')
        command = "".join(command).format(self.tmp_folder, self.root_project_folder, self.root_project_folder)
        output = subprocess.check_output(command, shell=True).decode()
        self.assertIn("os=Android", output)
        self.assertIn("compiler.version=8", output)
        self.assertIn("compiler=clang", output)
        self.assertIn("arch=x86_64", output)
        self.assertIn("Cross-build from 'Linux:x86_64' to 'Android:x86_64'", output)

    @unittest.skipUnless(is_linux_and_have_docker(), "Requires Linux and Docker")
    def test_docker_custom_pip_command(self):
        conanfile = """from conans import ConanFile
        import os

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "zlib/1.2.11@conan/stable"

            def build(self):
                pass
        """
        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_DOCKER_ENTRY_SCRIPT": "pip install -U /tmp/cpt",
                                       "CONAN_USERNAME": "bar",
                                       "CONAN_DOCKER_IMAGE": "conanio/gcc8",
                                       "CONAN_REFERENCE": "foo/0.0.1@bar/testing",
                                       "CONAN_DOCKER_RUN_OPTIONS": "--network=host, --add-host=google.com:8.8.8.8 -v{}:/tmp/cpt".format(
                                           self.root_project_folder),
                                       "CONAN_DOCKER_IMAGE_SKIP_UPDATE": "TRUE",
                                       "CONAN_FORCE_SELINUX": "TRUE",
                                       "CONAN_DOCKER_SHELL": "/bin/bash -c",
                                       "CONAN_DOCKER_PIP_COMMAND": "foobar"
                                       }):
            self.packager = ConanMultiPackager(gcc_versions=["8"],
                                               archs=["x86_64"],
                                               build_types=["Release"],
                                               out=self.output.write)
            self.packager.add({})
            with self.assertRaises(Exception) as raised:
                self.packager.run()
                self.assertIn("Error updating the image", str(raised.exception))
                self.assertIn("foobar install conan_package_tools", str(raised.exception))



import unittest

from conans.client.tools import environment_append
from conans.test.utils.tools import TestClient, TestServer
from cpt.test.unit.utils import MockCIManager
from cpt.test.test_client.tools import get_patched_multipackager



class UpdateTest(unittest.TestCase):

    conanfile_bar = """from conans import ConanFile
class Pkg(ConanFile):
    name = "bar"
    version = "0.1.0"

    def build(self):
        pass
    """

    conanfile_foo = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foo"
    version = "1.0.0"
    options = {"shared": [True, False]}
    default_options = "shared=True"

    def build(self):
        pass
    """

    conanfile_foo_2 = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foo"
    version = "1.0.0"
    options = {"shared": [True, False]}
    default_options = "shared=False"

    def build(self):
        self.output.info("new foo")
    """

    conanfile_foo_3 = """from conans import ConanFile
class Pkg(ConanFile):
    name = "qux"
    version = "1.0.0"
    options = {"shared": [True, False]}
    default_options = "shared=False"

    def build(self):
        self.output.info("qux")
    """

    conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "2.0"
    requires = "bar/0.1.0@foo/stable", "foo/1.0.0@bar/testing", "qux/1.0.0"

    def build(self):
        self.output.warn("BUILDING")
"""

    def setUp(self):
        self._ci_manager = MockCIManager()
        self._server = TestServer(users={"user": "password"},
                                  write_permissions=[("bar/0.1.0@foo/stable", "user"),
                                                     ("foo/1.0.0@bar/testing", "user"),
                                                     ("qux/1.0.0", "user")])
        self._client = TestClient(servers={"default": self._server},
                                  users={"default": [("user", "password")]})
        self._client.save({"conanfile_bar.py": self.conanfile_bar})
        self._client.run("export conanfile_bar.py foo/stable")
        self._client.save({"conanfile_foo.py": self.conanfile_foo})
        self._client.run("export conanfile_foo.py bar/testing")
        self._client.save({"conanfile_foo3.py": self.conanfile_foo_3})
        self._client.run("export conanfile_foo3.py")
        self._client.save({"conanfile.py": self.conanfile})

    def test_update_all_dependencies(self):
        with environment_append({"CONAN_UPLOAD":  self._server.fake_url,
                                 "CONAN_LOGIN_USERNAME": "user",
                                 "CONAN_PASSWORD": "password", "CONAN_USERNAME": "user",
                                 "CONAN_UPLOAD_DEPENDENCIES": "all",
                                 "CONAN_UPDATE_DEPENDENCIES": "True"}):

            mulitpackager = get_patched_multipackager(self._client, username="user",
                                                      channel="testing",
                                                      build_policy="missing",
                                                      exclude_vcvars_precommand=True,
                                                      ci_manager=self._ci_manager)
            mulitpackager.add({}, {})
            mulitpackager.run()

            self.assertIn("Uploading packages for 'foobar/2.0@user/testing'", self._client.out)
            self.assertIn("Uploading packages for 'bar/0.1.0@foo/stable'", self._client.out)
            self.assertIn("Uploading packages for 'foo/1.0.0@bar/testing'", self._client.out)
            self.assertIn("Uploading packages for 'qux/1.0.0@'", self._client.out)

            # Upload new version of foo/1.0.0@bar/testing and re-add old revision in local cache
            self._client.save({"conanfile_foo.py": self.conanfile_foo_2})
            mulitpackager = get_patched_multipackager(self._client, username="user",
                                                      channel="testing",
                                                      build_policy="missing",
                                                      exclude_vcvars_precommand=True,
                                                      ci_manager=self._ci_manager,
                                                      conanfile="conanfile_foo.py")
            mulitpackager.add({}, {})
            mulitpackager.run()

            self._client.run("remove -f foo/1.0.0@bar/testing")
            self._client.save({"conanfile_foo.py": self.conanfile_foo})
            self._client.run("export conanfile_foo.py bar/testing")

            # build again and update newest revision from remote
            mulitpackager = get_patched_multipackager(self._client, username="user",
                                                      channel="testing",
                                                      build_policy="missing",
                                                      exclude_vcvars_precommand=True,
                                                      ci_manager=self._ci_manager)
            mulitpackager.add({}, {})
            mulitpackager.run()
            self.assertIn("foo/1.0.0@bar/testing: Package installed", self._client.out)



import os
import tempfile

from conans.client import tools
from conans.client.profile_loader import _load_profile
from conans.model.version import Version
from conans.util.files import save
from cpt import get_client_version


def get_profiles(client_cache, build_config, base_profile_name=None):

    base_profile_text = ""
    if base_profile_name:
        base_profile_path = os.path.join(client_cache.profiles_path, base_profile_name)
        base_profile_text = tools.load(base_profile_path)
    base_profile_name = base_profile_name or "default"
    tmp = """
include(%s)

[settings]
%s
[options]
%s
[env]
%s
[build_requires]
%s
"""

    def pairs_lines(items):
        return "\n".join(["%s=%s" % (k, v) for k, v in items])

    settings = pairs_lines(sorted(build_config.settings.items()))
    options = pairs_lines(build_config.options.items())
    env_vars = pairs_lines(build_config.env_vars.items())
    br_lines = ""
    for pattern, build_requires in build_config.build_requires.items():
        br_lines += "\n".join(["%s:%s" % (pattern, br) for br in build_requires])

    if os.getenv("CONAN_BUILD_REQUIRES"):
        brs = os.getenv("CONAN_BUILD_REQUIRES").split(",")
        brs = ['*:%s' % br.strip() if ":" not in br else br for br in brs]
        if br_lines:
            br_lines += "\n"
        br_lines += "\n".join(brs)

    profile_text = tmp % (base_profile_name, settings, options, env_vars, br_lines)
    return profile_text, base_profile_text


def patch_default_base_profile(conan_api, profile_abs_path):
    """If we have a profile including default, but the users default in config is that the default
    is other, we have to change the include"""
    text = tools.load(profile_abs_path)
    if "include(default)" in text:  # User didn't specified a custom profile
        conan_version = get_client_version()
        if conan_version < Version("1.12.0"):
            cache = conan_api._client_cache
        elif conan_version < Version("1.18.0"):
            cache = conan_api._cache
        else:
            if not conan_api.app:
                conan_api.create_app()
            cache = conan_api.app.cache

        default_profile_name = os.path.basename(cache.default_profile_path)
        if not os.path.exists(cache.default_profile_path):
            conan_api.create_profile(default_profile_name, detect=True)

        if default_profile_name != "default":  # User have a different default profile name
            # https://github.com/conan-io/conan-package-tools/issues/121
            text = text.replace("include(default)", "include(%s)" % default_profile_name)
            tools.save(profile_abs_path, text)


def save_profile_to_tmp(profile_text):
    # Save the profile in a tmp file
    tmp = os.path.join(tempfile.mkdtemp(suffix='conan_package_tools_profiles'), "profile")
    abs_profile_path = os.path.abspath(tmp)
    save(abs_profile_path, profile_text)
    return abs_profile_path


def load_profile(profile_abs_path, client_cache):
    text = tools.load(profile_abs_path)
    profile, _ = _load_profile(text, os.path.dirname(profile_abs_path),
                               client_cache.profiles_path)
    return profile




import os
from six import string_types


class AuthManager(object):

    def __init__(self, conan_api, printer, login_input=None,
                 passwords_input=None, default_username=None, skip_check_credentials=False):
        """
        :param conan_api: ConanAPI
        :param login_input: Can be a string with the user or a dict with {"remote": "login"}
        :param passwords_input: Can be a string with the password or a dict with {"remote": "pass"}

        It also looks in the CONAN_LOGIN, CONAN_LOGIN_USERNAME, CONAN_USERNAME, CONAN_PASSWORD and
        CONAN_PASSWORD_XXXX
        """

        self._conan_api = conan_api
        self._data = {}  # {"remote_name": (user, password)}
        self.printer = printer
        self.skip_check_credentials = skip_check_credentials

        unique_login = self._get_single_login_username(login_input) or default_username
        unique_password = self._get_single_password(passwords_input)

        dict_login = self._get_multiple_logins(login_input)
        dict_password = self._get_multiple_passwords(passwords_input)

        if dict_login:
            for remote, username in dict_login.items():
                if remote not in dict_password:
                    raise Exception("Password for remote '%s' not specified" % remote)
                self._data[remote.lower()] = (username, dict_password[remote])
        elif unique_login:
            if not unique_password and dict_password:
                raise Exception("Specify a dict for 'login_username' or CONAN_LOGIN_USERNAME_XXX"
                                " for the login=%s" % unique_login)

            self._data[None] = (unique_login, unique_password)
        else:
            self.printer.print_message(str(os.environ))
            self._data[None] = (None, None)

    @staticmethod
    def _get_single_login_username(logins_input):
        if logins_input and isinstance(logins_input, string_types):
            return logins_input
        if os.getenv("CONAN_LOGIN_USERNAME"):
            return os.getenv("CONAN_LOGIN_USERNAME").replace('"', '\\"')
        if os.getenv("CONAN_USERNAME"):
            return os.getenv("CONAN_USERNAME").replace('"', '\\"')

    @staticmethod
    def _get_single_password(passwords_input):
        if passwords_input and isinstance(passwords_input, string_types):
            return passwords_input
        if not passwords_input and os.getenv("CONAN_PASSWORD"):
            return os.getenv("CONAN_PASSWORD").replace('"', '\\"')

    @staticmethod
    def _get_multiple_logins(logins_input):
        if logins_input and isinstance(logins_input, dict):
            return logins_input

        ret = {}
        for name in os.environ.keys():
            if name.startswith("CONAN_LOGIN_USERNAME_"):
                remote_name = name.split("_", 3)[3]
                ret[remote_name] = os.environ[name]
        return ret

    @staticmethod
    def _get_multiple_passwords(passwords_input):
        if passwords_input and isinstance(passwords_input, dict):
            return passwords_input

        ret = {}
        for name in os.environ.keys():
            if name.startswith("CONAN_PASSWORD_"):
                remote_name = name.split("_", 2)[2]
                ret[remote_name] = os.environ[name].replace('"', '\\"')
        return ret

    def get_user_password(self, remote=None):
        if remote:
            remote = remote.lower()
        if None in self._data:  # General user and password for the same remote
            return self._data[None]
        if remote not in self._data:
            raise Exception("User and password for remote '%s' not specified" % remote)
        return self._data[remote]

    def credentials_ready(self, upload_remote_name):
        user, password = self.get_user_password(upload_remote_name)
        return (user and password) or self.skip_check_credentials

    def login(self, remote_name):
        self.printer.print_message("Verifying credentials...")
        user, password = self.get_user_password(remote_name)
        if not (user and password) and self.skip_check_credentials:  # Assume that it is already logged.
                self.printer.print_message("Credentials not specified but 'skip_check_credentials' "
                                           "activated, trying to use pre-stored user/password in "
                                           "local cache")
                return

        self._conan_api.authenticate(user, password, remote_name)
        self.printer.print_message("OK! '%s' user logged in '%s' " % (user, remote_name))

    def env_vars(self):
        ret = {}
        if None in self._data:
            username, password = self._data[None]
            if username:
                ret["CONAN_LOGIN_USERNAME"] = username
            if password:
                ret["CONAN_PASSWORD"] = password
            return ret

        for remote, (login, password) in self._data.items():
            ret["CONAN_LOGIN_USERNAME_%s" % remote.upper()] = login
            ret["CONAN_PASSWORD_%s" % remote.upper()] = password

        return ret



import os
import unittest

from conans.util.files import mkdir_tmp

from conans import tools
from conans.client.conan_api import ConanAPIV1
from conans.test.utils.tools import TestBufferConanOutput

CONAN_UPLOAD_URL = os.getenv("CONAN_UPLOAD_URL",
                             "https://conan.jfrog.io/conan/api/conan/conan-testsuite")
CONAN_UPLOAD_PASSWORD = os.getenv("CONAN_UPLOAD_PASSWORD", "")
CONAN_LOGIN_UPLOAD = os.getenv("CONAN_LOGIN_UPLOAD", "")


class BaseTest(unittest.TestCase):

    def setUp(self):
        self.old_folder = os.getcwd()
        self.tmp_folder = mkdir_tmp()
        os.chmod(self.tmp_folder, 0o777)
        self.conan_home = self.tmp_folder
        os.chdir(self.tmp_folder)
        # user_home = "c:/tmp/home"  # Cache
        self.old_env = dict(os.environ)
        os.environ.update({"CONAN_USER_HOME": self.conan_home, "CONAN_PIP_PACKAGE": "0"})
        self.output = TestBufferConanOutput()
        self.api, _, _ = ConanAPIV1.factory()
        self.api.create_app()
        self.client_cache = self.api.app.cache

    def tearDown(self):
        os.chdir(self.old_folder)
        os.environ.clear()
        os.environ.update(self.old_env)

    def save_conanfile(self, conanfile):
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)

    def create_project(self):
        with tools.chdir(self.tmp_folder):
            self.api.new("hello/0.1.0", pure_c=True)

    @property
    def root_project_folder(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        for i in range(10):
            if "setup.py" in os.listdir(dir_path):
                return dir_path
            else:
                dir_path = os.path.abspath(os.path.join(dir_path, os.pardir))
        raise Exception("Cannot find root project folder")





import unittest

from conans.client.tools import environment_append
from conans.test.utils.tools import TestClient, TestServer

from cpt.test.test_client.tools import get_patched_multipackager


class InvalidConfigTest(unittest.TestCase):

    conanfile = """from conans import ConanFile
from conans.errors import ConanInvalidConfiguration
class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "arch"

    def configure(self):
        if self.settings.arch == "x86":
            raise ConanInvalidConfiguration("This library doesn't support x86")
"""

    def test_invalid_configuration_skipped_but_warned(self):

        ts = TestServer(users={"user": "password"})
        tc = TestClient(servers={"default": ts}, users={"default": [("user", "password")]})
        tc.save({"conanfile.py": self.conanfile})
        with environment_append({"CONAN_UPLOAD": ts.fake_url, "CONAN_LOGIN_USERNAME": "user",
                                 "CONAN_PASSWORD": "password", "CONAN_USERNAME": "user"}):
            mulitpackager = get_patched_multipackager(tc, exclude_vcvars_precommand=True)
            mulitpackager.add({"arch": "x86_64"}, {})
            mulitpackager.add({"arch": "x86"}, {})
            mulitpackager.run()
            self.assertIn("Uploading package 1/1", tc.out)
            self.assertIn("Invalid configuration: This library doesn't support x86", tc.out)



import unittest
from cpt.packager import ConanMultiPackager


class ConanVersionTest(unittest.TestCase):

    def test_conan_incompatible_version(self):
        cpt = ConanMultiPackager(username="user", reference="lib/1.0@conan/stable")
        cpt._newest_supported_conan_version = "2.0"
        cpt._client_conan_version = "2.1"
        with self.assertRaisesRegexp(Exception, "Conan/CPT version mismatch. "
                                                "Conan version installed: 2.1 . "
                                                "This version of CPT supports only Conan < 2.0"):
            cpt.run()

        cpt._newest_supported_conan_version = "2.1"
        cpt._client_conan_version = "2.1"
        cpt.run()
